settings.outformat="png";
settings.render=16;
import three;
import math;
size(8cm,0);

currentprojection = orthographic((-2.27827101703374,-14.9375073092961,1.57835554922396));

// draw(O--2X ^^ O--2Y ^^ O--2Z); // Axes

// Common drawing things
pen dot_line = dotted+linewidth(0.5);
pen sld_line = solid+linewidth(0.5);


// Common camera parameters

// Some common parameters for rendering the representation of the camera and image planes
// This assumes a nominal position of the camera. Things are updated later for the specific instances
// when we move the camera in space
real f = 1;           // Camera focal length
triple prinPt = f*Z;  // The nominal location of the principle point on the Z axis
triple optAxis = Z;   // Vector describing direction of optical axis (unrotated). Normal to the image plane.

real width = 2; // The width of the plane - just for drawing the representation
real height = 1; // The height of the plane - just for drawing the representation
triple planeOffset = (-width/2, -height/2, f);
triple planeVectorsX = (width, 0, 0);
triple planeVectorsY = (0, height, 0);
path3 imgPl = plane(planeVectorsX,planeVectorsY,planeOffset);

// Parameters of the first camera - basically the extrinsics
triple C1 = (0,0,0); // Translation of the camera
transform3 rot1 = rotate(-50,Z)*rotate(-90,X);
transform3 trn1 = shift(C1);
triple optAxis1 = rot1*optAxis;  // Together, the principle point and the optical axis describe the image plane
triple prinPt1  = trn1*rot1*prinPt;

path3 imgPl1 = trn1*rot1*imgPl;  // Get a path that describes a nominal bounds of the plane (so we can draw it)
draw(imgPl1);
dot(C1,L=Label("$\mathbf{C}$",Relative(2*S),p=fontsize(5pt)));

// Parameters of the second camera - basically the extrinsics
triple C2 = (4,0,0); // Translation of the camera
transform3 rot2 = rotate(50,Z)*rotate(-90,X);
transform3 trn2 = shift(C2);
triple optAxis2 = rot2*optAxis;  // Together, the principle point and the optical axis describe the image plane
triple prinPt2  = trn2*rot2*prinPt;
path3 imgPl2 = trn2*rot2*imgPl;  // Get a path that describes a nominal bounds of the plane (so we can draw it)

draw(imgPl2);
dot(C2,L=Label("$\mathbf{C}'$",Relative(2*S),p=fontsize(5pt)));

// Get the epipolar points
triple e1  = point(C1--C2,intersect(C1,C2,optAxis1,prinPt1));
triple e2  = point(C1--C2,intersect(C1,C2,optAxis2,prinPt2));

dot(e1,L=Label("$\mathbf{e}$",Relative(2*S),p=fontsize(5pt)));
dot(e2,L=Label("$\mathbf{e}'$",Relative(2*S),p=fontsize(5pt)));

// Points in 3D
triple X1 = (2,4,0.5);
triple X2 = (2.5,4.4,1.0);

dot(X1,L=Label("$\mathbf{P}$",Relative(2*N),p=fontsize(5pt)));
dot(X2,L=Label("$\mathbf{P}'$",Relative(2*N),p=fontsize(5pt)));
draw(point(X1--X2,0.1)--point(X1--X2,0.9),arrow=Arrow3(TeXHead2));
draw(X1--C1,dot_line);
draw(X2--C2,dot_line);

// Project the points to the planes
triple x1  = point(C1--X1,intersect(C1,X1,optAxis1,prinPt1));
triple x2  = point(C2--X2,intersect(C2,X2,optAxis2,prinPt2));
triple x1d = point(C2--X1,intersect(C2,X1,optAxis2,prinPt2));

dot(x1,L=Label("$\mathbf{p}$",Relative(2*NW),p=fontsize(5pt)));
dot(x2,L=Label("$\mathbf{p}'$",Relative(2*NE),p=fontsize(5pt)));

// Epipolar
triple EpiNorm = unit(cross(C2-C1,x1-C1)); // Epipolar plane

draw(C1--C2,dot_line);

// Get the image of X1 on plane2, which is also on the epipolar line
// Extend the point
triple extendPt1 = e2 + 2*(x1d - e2);
triple extendPt2 = e2 - 2*(x1d - e2);
path3 extendedEpiLine = extendPt1--extendPt2;
triple[] epiInt = intersectionpoints(imgPl2,extendedEpiLine);
path3 boundedEpiLine = epiInt[0]--epiInt[1];

draw(boundedEpiLine,sld_line,L=Label("$\mathbf{l}'$",Relative(2*NW),p=fontsize(5pt)));


// Get projection of point onto epipolar line
real ratio = dot(unit(x2-e2),unit(x1d-e2));
real dist = ratio*length(x2-e2);
triple e = e2 + dist*unit(x1d-e2);
path3 d = x2--e;

draw(d,L=Label("$d$",Relative(2*W),p=fontsize(5pt)));
