settings.outformat="png";
settings.render=16;
import three;
import solids;
import math;
size(8cm,0);

currentprojection = orthographic((13.2883877152396,2.79596754341699,6.81262373945653),up=Y);

//draw(O--2X ^^ O--2Y ^^ O--2Z); // Axes
//label("$X$",2X);
//label("$Y$",2Y);
//label("$Z$",2Z);

// Common drawing things
pen dot_line = dotted+linewidth(0.5);
pen sld_line = solid+linewidth(0.5);


// Common camera parameters

// Some common parameters for rendering the representation of the camera and image planes
// This assumes a nominal position of the camera. Things are updated later for the specific instances
// when we move the camera in space
real f = 2;           // Camera focal length
triple prinPt = -f*Z;  // The nominal location of the principle point on the Z axis
triple optAxis = Z;   // Vector describing direction of optical axis (unrotated). Normal to the image plane.

real width = 3; // The width of the plane - just for drawing the representation
real height = 1.5; // The height of the plane - just for drawing the representation
triple planeOffset = (-width/2, -height/2, -f);
triple planeVectorsX = (width, 0, 0);
triple planeVectorsY = (0, height, 0);
path3 imgPl = plane(planeVectorsX,planeVectorsY,planeOffset);

// Parameters the first camera - basically the extrinsics
triple C1 = (0,0,0); // Translation of the camera
transform3 rot1 = rotate(0,Z)*rotate(0,X);
transform3 trn1 = shift(C1);
triple optAxis1 = rot1*optAxis;  // Together, the principle point and the optical axis describe the image plane
triple prinPt1  = trn1*rot1*prinPt;

path3 imgPl1 = trn1*rot1*imgPl;  // Get a path that describes a nominal bounds of the plane (so we can draw it)
draw(imgPl1,L=Label("$\mathbf{I}$",Relative(2*N),p=fontsize(5pt)));
dot(C1,L=Label("$\mathbf{C}$",Relative(2*S),p=fontsize(5pt)));
dot(prinPt1,L=Label("$\mathbf{o}$",Relative(2*E),p=fontsize(5pt)));
draw(prinPt1--C1--(1.2*(C1-prinPt1)),dashed);

// The unit sphere
revolution sph=sphere((0,0,0), r=1, n=10);
surface ssph = surface(sph);

draw(ssph, surfacepen=white+opacity(0), meshpen=black+linewidth(0.3));
label("$\mathbf{S}^2$", point(C1--(C1+2*Y),intersect(C1--(C1+2*Y),ssph)[0]*1.1),p=fontsize(5pt));

// Points
triple P = (-1, 1, 2.5);
triple p = point(P--C1,intersect(P--C1,ssph)[0]);
triple ud = point(P--(C1-2*P),intersect(P,C1-2*P,optAxis1,prinPt1));
triple u = (ud.x*0.4, ud.y*0.4, ud.z); // The scaling of x an y mocks the fisheye distortion

dot(u,L=Label("$\mathbf{u}$",Relative(2*S),p=fontsize(5pt)));
//dot(ud,L=Label("$\mathbf{u_d}$",Relative(2*S),p=fontsize(5pt)));
dot(p,L=Label("$\mathbf{p}$",Relative(2*N),p=fontsize(5pt)));
dot(P,L=Label("$\mathbf{P}$",Relative(2*S),p=fontsize(5pt)));
draw(P--C1,dotted);
draw(C1--u,dotted);
draw(prinPt1--u,L=Label("$r$",p=fontsize(5pt)));

// Incident angle
path3 th = arc(C1,0.5*Z,0.5*p);
draw(th,L=Label("$\theta$",p=fontsize(5pt)));



